@@ -211,7 +211,7 @@ QList<QueryMatch> DBusRunner::convertMatches(const QString &service, const Remot
         if (iconData.isValid()) {
             const auto iconDataArgument = iconData.value<QDBusArgument>();
             if (iconDataArgument.currentType() == QDBusArgument::StructureType && iconDataArgument.currentSignature() == QLatin1String("(iiibiiay)")) {
-                const RemoteImage remoteImage = qdbus_cast<RemoteImage>(iconDataArgument);
+                const auto remoteImage = qdbus_cast<RemoteImage>(iconDataArgument);
                 if (QImage decodedImage = decodeImage(remoteImage); !decodedImage.isNull()) {
                     const QPixmap pix = QPixmap::fromImage(std::move(decodedImage));
                     QIcon icon(pix);
@@ -289,23 +289,15 @@ void DBusRunner::run(const KRunner::RunnerContext & /*context*/, const KRunner::
     };
 
     if (KWindowSystem::isPlatformWayland() && qGuiApp->focusWindow()) {
-        const int launchedSerial = KWaylandExtras::lastInputSerial(qGuiApp->focusWindow());
-        connect(
-            KWaylandExtras::self(),
-            &KWaylandExtras::xdgActivationTokenArrived,
-            this,
-            [this, launchedSerial, service, matchId, actionId, run](int tokenSerial, const QString &token) {
-                Q_UNUSED(tokenSerial);
-                if (!token.isEmpty()) {
-                    auto activationTokenMethod = QDBusMessage::createMethodCall(service, m_path, m_ifaceName, QStringLiteral("SetActivationToken"));
-                    activationTokenMethod.setArguments(QList<QVariant>{token});
-                    QDBusConnection::sessionBus().call(activationTokenMethod, QDBus::NoBlock);
-                }
-
-                run();
-            },
-            Qt::SingleShotConnection);
-        KWaylandExtras::requestXdgActivationToken(qGuiApp->focusWindow(), launchedSerial, {});
+        auto tokenFuture = KWaylandExtras::xdgActivationToken(qGuiApp->focusWindow(), {});
+        tokenFuture.then(this, [this, service, matchId, actionId, run](const QString &token) {
+            if (!token.isEmpty()) {
+                auto activationTokenMethod = QDBusMessage::createMethodCall(service, m_path, m_ifaceName, QStringLiteral("SetActivationToken"));
+                activationTokenMethod.setArguments(QList<QVariant>{token});
+                QDBusConnection::sessionBus().call(activationTokenMethod, QDBus::NoBlock);
+            }
+            run();
+        });
     } else {
         run();
     }
@@ -330,7 +322,7 @@ QImage DBusRunner::decodeImage(const RemoteImage &remoteImage)
     if (remoteImage.width <= 0 || remoteImage.width >= 2048 || remoteImage.height <= 0 || remoteImage.height >= 2048 || remoteImage.rowStride <= 0) {
         qCWarning(KRUNNER) << "Invalid image metadata (width:" << remoteImage.width << "height:" << remoteImage.height << "rowStride:" << remoteImage.rowStride
                            << ")";
-        return QImage();
+        return {};
     }
 
     QImage::Format format = QImage::Format_Invalid;
@@ -347,7 +339,7 @@ QImage DBusRunner::decodeImage(const RemoteImage &remoteImage)
     if (format == QImage::Format_Invalid) {
         qCWarning(KRUNNER) << "Unsupported image format (hasAlpha:" << remoteImage.hasAlpha << "bitsPerSample:" << remoteImage.bitsPerSample
                            << "channels:" << remoteImage.channels << ")";
-        return QImage();
+        return {};
     }
 
     QImage image(remoteImage.width, remoteImage.height, format);
